# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

from django.core.exceptions import ImproperlyConfigured
from django.db.models.query import QuerySet
from django.utils.functional import wraps

try:
    from django.db.transaction import atomic
except ImportError:
    from django.db.transaction import commit_on_success as atomic


def list_chunks_iterator(l, n):
    """ Yield successive n-sized chunks from l.
    """
    for i in xrange(0, len(l), n):
        yield l[i:i+n]


def opt_arguments(func):
    '''
    Meta-decorator for ablity use decorators with optional arguments
    from here http://www.ellipsix.net/blog/2010/08/more-python-voodoo-optional-argument-decorators.html
    '''
    def meta_wrapper(*args, **kwargs):
        if len(args) == 1 and callable(args[0]):
            # No arguments, this is the decorator
            # Set default values for the arguments
            return func(args[0])
        else:
            def meta_func(inner_func):
                return func(inner_func, *args, **kwargs)
            return meta_func
    return meta_wrapper


@opt_arguments
def fetch_all(func, return_all=None, always_all=False, pagination='anchor', has_more='has_more'):
    """
    Class method decorator for fetching all items. Add parameter `all=False` for decored method.
    If `all` is True, method runs as many times as it returns any results.
    Decorator receive parameters:
      * callback method `return_all`. It's called with the same parameters
        as decored method after all itmes are fetched.
      * `always_all` bool - return all instances in any case of argument `all`
        of decorated method
    Usage:

        @fetch_all(return_all=lambda self,instance,*a,**k: instance.items.all())
        def fetch_something(self, ..., *kwargs):
        ....
    """
    def wrapper(self, all=False, instances_all=None, *args, **kwargs):

        if len(args) > 0:
            raise ImproperlyConfigured("It's prohibited to use non-key arguments for method decorated with @fetch_all,"
                                       " method is %s.%s(), args=%s" % (self.__class__.__name__, func.__name__, args))

        response = {}
        instances = func(self, **kwargs)
        if len(instances) == 2 and isinstance(instances, tuple):
            instances, response = instances

        if always_all or all:
            if isinstance(instances, QuerySet):
                if instances_all is None:
                    instances_all = instances.none()
                instances_count = instances.count()
                if instances_count:
                    instances_all |= instances
            elif isinstance(instances, list):
                if instances_all is None:
                    instances_all = []
                instances_count = len(instances)
                instances_all += instances
            else:
                raise ValueError("Wrong type of response from func %s. It should be QuerySet or list, "
                                 "not a %s" % (func, type(instances)))

            # recursive pagination
            # print has_more, response.keys()
            if instances_count and (has_more in response and response[has_more]
                                    or has_more not in response and pagination in response):
                kwargs[pagination] = response.get(pagination)
                return wrapper(self, all=all, instances_all=instances_all, **kwargs)

            if return_all:
                kwargs['instances'] = instances_all
                return return_all(self, **kwargs)
            else:
                return instances_all
        else:
            return instances

    return wraps(func)(wrapper)


@opt_arguments
def fetch_only_expired(func, timeout_days, expiration_fieldname='fetched', ids_argument='ids'):
    """
    Class method decorator for fetching only expired items. Add parameter `only_expired=False` for decored method.
    If `only_expired` is True, method substitute argument `ids_argument` with new value, that consist only expired ids.
    Decorator receive parameters:
      * `timeout_days` int, number of day, after that instance is suppose to be expired.
      * `expiration_fieldname` string, name of datetime field, that indicate time of instance last fetching
      * `ids_argument` string, name of argument, that store list of ids.
    Usage:

        @fetch_only_expired(timeout_days=3)
        def fetch_something(self, ..., *kwargs):
        ....
    """
    def wrapper(self, only_expired=False, *args, **kwargs):

        if len(args) > 0:
            raise ValueError("It's prohibited to use non-key arguments for method decorated with @fetch_all, "
                             "method is %s.%s(), args=%s" % (self.__class__.__name__, func.__name__, args))

        if only_expired:
            ids = kwargs[ids_argument]
            expired_at = datetime.now() - timedelta(timeout_days)
            ids_non_expired = self.model.objects.filter(**{'%s__gte' % expiration_fieldname: expired_at,
                                                           'pk__in': ids}).values_list('pk', flat=True)
            kwargs[ids_argument] = list(set(ids).difference(set(ids_non_expired)))

            instances = None
            if len(kwargs[ids_argument]):
                instances = func(self, **kwargs)
            return renew_if_not_equal(self.model, instances, ids)

        return func(self, **kwargs)

    return wraps(func)(wrapper)


@opt_arguments
def fetch_by_chunks_of(func, items_limit, ids_argument='ids'):
    """
    Class method decorator for fetching ammount of items bigger than allowed at once.
    Decorator receive parameters:
      * `items_limit`. Max limit of allowned items to fetch at once
      * `ids_argument` string, name of argument, that store list of ids.
    Usage:

        @fetch_by_chunks_of(1000)
        def fetch_something(self, ..., *kwargs):
        ....
    """
    def wrapper(self, *args, **kwargs):

        if len(args) > 0:
            raise ValueError("It's prohibited to use non-key arguments for method decorated with @fetch_all, "
                             "method is %s.%s(), args=%s" % (self.__class__.__name__, func.__name__, args))

        ids = kwargs[ids_argument]
        if ids:
            kwargs_sliced = dict(kwargs)
            for chunk in list_chunks_iterator(ids, items_limit):
                kwargs_sliced[ids_argument] = chunk
                instances = func(self, **kwargs_sliced)

            return renew_if_not_equal(self.model, instances, ids)
        else:
            return func(self, **kwargs)

    return wraps(func)(wrapper)


def renew_if_not_equal(model, instances, ids):
    return instances if instances is not None and len(ids) == instances.count() else model.objects.filter(pk__in=ids)


def opt_generator(func):
    """
    Class method or function decorator makes able to call generator methods as usual methods.
    Usage:

        @method_decorator(opt_generator)
        def some_method(self, ...):
            ...
            for count in some_another_method():
                yield (count, total)

    It's possible to call this method 2 different ways:

        * instance.some_method() - it will return nothing
        * for count, total in instance.some_method(as_generator=True):
            print count, total
    """
    def wrapper(*args, **kwargs):
        as_generator = kwargs.pop('as_generator', False)
        result = func(*args, **kwargs)
        return result if as_generator else list(result)
    return wraps(func)(wrapper)



# -*- coding: utf-8 -*-
from django.conf import settings
from oauth_tokens.api import ApiAbstractBase, Singleton
from odnoklassniki import api, OdnoklassnikiError
from simplejson.decoder import JSONDecodeError

__all__ = ['api_call', 'OdnoklassnikiError']

APPLICATION_PUBLIC = getattr(settings, 'OAUTH_TOKENS_ODNOKLASSNIKI_CLIENT_PUBLIC', '')
APPLICATION_SECRET = getattr(settings, 'OAUTH_TOKENS_ODNOKLASSNIKI_CLIENT_SECRET', '')


class OdnoklassnikiApi(ApiAbstractBase):

    __metaclass__ = Singleton

    provider = 'odnoklassniki'
    error_class = OdnoklassnikiError
    error_class_repeat = tuple(list(ApiAbstractBase.error_class_repeat) + [JSONDecodeError])

    def get_consistent_token(self):
        return getattr(settings, 'ODNOKLASSNIKI_API_ACCESS_TOKEN', None)

    def get_api(self, token):
        return api.Odnoklassniki(application_key=APPLICATION_PUBLIC, application_secret=APPLICATION_SECRET, token=token)

    def get_api_response(self, *args, **kwargs):
        return self.api._get(self.method, *args, **kwargs)

    def handle_error_code(self, e, *args, **kwargs):
        if e.code is None and e.message == 'HTTP error':
            return self.sleep_repeat_call(*args, **kwargs)
        else:
            return super(OdnoklassnikiApi, self).handle_error_code(e, *args, **kwargs)

    def handle_error_code_2(self, e, *args, **kwargs):
        # SERVICE : Service is temporary unavailable., logged by ID :
        return self.sleep_repeat_call(*args, **kwargs)

    def handle_error_code_8(self, e, *args, **kwargs):
        # FLOOD_BLOCKED : Call blocked due to flood protection
        return self.sleep_repeat_call(*args, **kwargs)

    def handle_error_code_102(self, e, *args, **kwargs):
        # PARAM_SESSION_EXPIRED : Session expired
        self.refresh_tokens()
        return self.repeat_call(*args, **kwargs)


def api_call(*args, **kwargs):
    api = OdnoklassnikiApi()
    return api.call(*args, **kwargs)



import os
import sys
import argparse
from django.conf import settings

class QuickDjangoTest(object):
    """
    A quick way to run the Django test suite without a fully-configured project.

    Example usage:

        >>> QuickDjangoTest('app1', 'app2')

    Based on a script published by Lukasz Dziedzia at:
    http://stackoverflow.com/questions/3841725/how-to-launch-tests-for-django-reusable-app
    """
    DIRNAME = os.path.dirname(__file__)
    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
    )

    def __init__(self, *args, **kwargs):
        self.apps = args
        # Get the version of the test suite
        self.version = self.get_test_version()
        # Call the appropriate one
        if self.version == 'new':
            self._new_tests()
        else:
            self._old_tests()

    def get_test_version(self):
        """
        Figure out which version of Django's test suite we have to play with.
        """
        from django import VERSION
        if VERSION[0] == 1 and VERSION[1] >= 2:
            return 'new'
        else:
            return 'old'

    def _old_tests(self):
        """
        Fire up the Django test suite from before version 1.2
        """
        INSTALLED_APPS, settings_test = self.custom_settings()

        settings.configure(DEBUG = True,
            DATABASE_ENGINE = 'sqlite3',
            DATABASE_NAME = os.path.join(self.DIRNAME, 'database.db'),
            INSTALLED_APPS = self.INSTALLED_APPS + INSTALLED_APPS + self.apps,
            **settings_test
        )
        from django.test.simple import run_tests
        failures = run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _new_tests(self):
        """
        Fire up the Django test suite developed for version 1.2
        """
        INSTALLED_APPS, settings_test = self.custom_settings()

        test_db = os.environ.get('DB', 'sqlite')
        if test_db == 'mysql':
            database = {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': 'django',
                'USER': 'root',
            }
        elif test_db == 'postgres':
            database = {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'USER': 'postgres',
                'NAME': 'django',
                'OPTIONS': {
                    'autocommit': True,
                }
            }
        else:
            database = {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(self.DIRNAME, 'database.db'),
                'USER': '',
                'PASSWORD': '',
                'HOST': '',
                'PORT': '',
            }

        settings.configure(
            DEBUG = True,
            DATABASES = {
                'default': database,
            },
            INSTALLED_APPS = self.INSTALLED_APPS + INSTALLED_APPS + self.apps,
            **settings_test
        )

        from django.test.simple import DjangoTestSuiteRunner
        failures = DjangoTestSuiteRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def custom_settings(self):
        try:
            from settings_test import *
            settings_test = dict(locals())
            del settings_test['self']
            if 'INSTALLED_APPS' in settings_test:
                del settings_test['INSTALLED_APPS']
        except ImportError:
            settings_test = {}
            INSTALLED_APPS = tuple()

        return INSTALLED_APPS, settings_test

if __name__ == '__main__':
    """
    What do when the user hits this file from the shell.

    Example usage:

        $ python quicktest.py app1 app2

    """
    parser = argparse.ArgumentParser(
        usage="[args]",
        description="Run Django tests on the provided applications."
    )
    parser.add_argument('apps', nargs='+', type=str)
    args = parser.parse_args()
    QuickDjangoTest(*args.apps)


# -*- coding: utf-8 -*-
import logging
import re
from abc import abstractmethod
from datetime import date, datetime

import pytz
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import IntegrityError, models, transaction
from django.db.models.fields import FieldDoesNotExist
from django.db.models.query import QuerySet
from django.utils import timezone
from django.utils.six import string_types

from . import fields
from .api import OdnoklassnikiError, api_call
from .decorators import atomic
from .fields_api import API_REQUEST_FIELDS

log = logging.getLogger('odnoklassniki_api')

COMMIT_REMOTE = getattr(settings, 'ODNOKLASSNIKI_API_COMMIT_REMOTE', True)
MASTER_DATABASE = getattr(settings, 'ODNOKLASSNIKI_API_MASTER_DATABASE', 'default')


class OdnoklassnikiDeniedAccessError(Exception):
    pass


class OdnoklassnikiContentError(Exception):
    pass


class OdnoklassnikiParseError(Exception):
    pass


class OdnoklassnikiManager(models.Manager):

    '''
    Odnoklassniki Ads API Manager for RESTful CRUD operations
    '''
    fields = API_REQUEST_FIELDS

    def get_request_fields(self, *args, **kwargs):
        fields = []
        for arg in args:
            for field in self.fields.get(arg, ''):
                if kwargs.get('prefix', False):
                    field = '%s.%s' % (arg, field)
                fields += [field]
        return ','.join(fields)

    def __init__(self, methods=None, remote_pk=None, *args, **kwargs):
        if methods and len(methods.items()) < 1:
            raise ValueError('Argument methods must contains at least 1 specified method')

        self.methods = methods or {}
        self.remote_pk = remote_pk or ('id',)

        super(OdnoklassnikiManager, self).__init__(*args, **kwargs)

    def get_by_url(self, url):
        '''
        Return existed User, Group, Application by url or new intance with empty pk
        '''
        m = re.findall(r'^(?:https?://)?(?:www.)?(?:ok.ru|odnoklassniki.ru)/(.+)/?$', url)
        if not len(m):
            raise ValueError("Wrong domain: %s" % url)

        slug = m[0]

        try:
            assert self.model.slug_prefix and slug.startswith(self.model.slug_prefix)
            id = int(re.findall(r'^%s(\d+)$' % self.model.slug_prefix, slug)[0])
        except (AssertionError, ValueError, IndexError):
            try:
                response = api_call('url.getInfo', url=url)
                assert self.model.resolve_screen_name_type == response['type']
                id = int(response['objectId'])
            except OdnoklassnikiError, e:
                log.error("Method get_by_slug returned error instead of response. URL='%s'. Error: %s" % (url, e))
                return None
            except (KeyError, TypeError, ValueError), e:
                log.error("Method get_by_slug returned response in strange format: %s. URL='%s'" % (response, url))
                return None
            except AssertionError:
                log.error("Method get_by_slug returned instance with wrong type '%s', not '%s'. URL='%s'" %
                          (response['type'], self.model.resolve_screen_name_type, url))
                return None

        try:
            object = self.model.objects.get(id=id)
        except self.model.DoesNotExist:
            object = self.model(id=id)  # , shortname=slug)

        return object

    def get_or_create_from_instances_list(self, instances):
        # python 2.6 compatibility
        # return
        # self.model.objects.filter(pk__in={self.get_or_create_from_instance(instance).pk
        # for instance in instances})
        return self.model.objects.filter(pk__in=set([self.get_or_create_from_instance(instance).pk for instance in instances]))

    def get_or_create_from_resources_list(self, response_list, extra_fields=None):
        instances = self.parse_response_list(response_list, extra_fields)
        return self.get_or_create_from_instances_list(instances)

    def get_or_create_from_instance(self, instance):

        remote_pk_dict = {}
        for field_name in self.remote_pk:
            remote_pk_dict[field_name] = getattr(instance, field_name)

        if remote_pk_dict:
            try:
                old_instance = self.model.objects.using(MASTER_DATABASE).get(**remote_pk_dict)
                instance._substitute(old_instance)
                instance.save()
            except self.model.DoesNotExist:
                instance.save()
                log.debug('Fetch and create new object %s with remote pk %s' % (self.model, remote_pk_dict))
        else:
            instance.save()
            log.debug('Fetch and create new object %s without remote pk' % (self.model,))

        return instance

    def get_or_create_from_resource(self, resource):

        instance = self.model()
        instance.parse(dict(resource))

        return self.get_or_create_from_instance(instance)

    def api_call(self, method='get', **kwargs):
        if self.model.methods_access_tag:
            kwargs['methods_access_tag'] = self.model.methods_access_tag

        method = self.methods[method]
        if self.model.methods_namespace:
            method = self.model.methods_namespace + '.' + method

        return api_call(method, **kwargs)

    def fetch_one(self, *args, **kwargs):
        return self.fetch(*args, **kwargs)

    @atomic
    def fetch(self, *args, **kwargs):
        '''
        Retrieve and save object to local DB
        '''
        result = self.get(*args, **kwargs)
        if isinstance(result, list):
            return self.get_or_create_from_instances_list(result)
        elif isinstance(result, QuerySet):
            return result
        else:
            return self.get_or_create_from_instance(result)

    def get(self, *args, **kwargs):
        '''
        Retrieve objects from remote server
        TODO: rename everywhere extra_fields to _extra_fields
        '''
        extra_fields = kwargs.pop('extra_fields', {})
        extra_fields['fetched'] = datetime.utcnow().replace(tzinfo=timezone.utc)

        self.response = self.api_call(*args, **kwargs)

        return self.parse_response(self.response, extra_fields)

    def parse_response(self, response, extra_fields=None):
        if isinstance(response, (list, tuple)):
            return self.parse_response_list(response, extra_fields)
        elif isinstance(response, dict):
            return self.parse_response_dict(response, extra_fields)
        else:
            raise OdnoklassnikiContentError('Odnoklassniki response should be list or dict, not %s' % response)

    # TODO: rename to parse_response_object
    def parse_response_dict(self, resource, extra_fields=None):

        instance = self.model()
        # important to do it before calling parse method
        if extra_fields:
            instance.__dict__.update(extra_fields)
        instance.parse(resource)

        return instance

    def parse_response_list(self, response_list, extra_fields=None):

        instances = []
        for resource in response_list:

            # in response with stats there is extra array inside each element
            if isinstance(resource, list) and len(resource):
                resource = resource[0]

            # in some responses first value is `count` of all values:
            # http://vk.com/developers.php?oid=-1&p=groups.search
            if isinstance(resource, int):
                continue

            try:
                resource = dict(resource)
            except (TypeError, ValueError), e:
                log.error("Resource %s is not dictionary" % resource)
                raise e

            instance = self.parse_response_dict(resource, extra_fields)
            instances += [instance]

        return instances


class OdnoklassnikiTimelineManager(OdnoklassnikiManager):

    '''
    Manager class, child of OdnoklassnikiManager for fetching objects with arguments `after`, `before`
    '''
    timeline_cut_fieldname = 'date'
    timeline_force_ordering = True

    def get_timeline_date(self, instance):
        return getattr(instance, self.timeline_cut_fieldname, datetime(1970, 1, 1).replace(tzinfo=timezone.utc))

    @atomic
    def get(self, *args, **kwargs):
        '''
        Retrieve objects and return result list with respect to parameters:
         * 'after' - excluding all items before.
         * 'before' - excluding all items after.
        '''
        after = kwargs.pop('after', None)
        before = kwargs.pop('before', None)

        if before and not after:
            raise ValueError("Attribute `before` should be specified with attribute `after`")
        if before and before < after:
            raise ValueError("Attribute `before` should be later, than attribute `after`")

        result = super(OdnoklassnikiTimelineManager, self).get(*args, **kwargs)

        if self.timeline_force_ordering and result:
            result.sort(key=self.get_timeline_date, reverse=True)

        instances = []
        for instance in result:

            timeline_date = self.get_timeline_date(instance)

            if timeline_date and isinstance(timeline_date, datetime):

                if after and after > timeline_date:
                    break

                if before and before < timeline_date:
                    continue

            instances += [instance]

        return instances


class OdnoklassnikiModel(models.Model):

    class Meta:
        abstract = True

    resolve_screen_name_type = ''
    remote_pk_field = 'id'
    remote_pk_local_field = 'id'
    methods_access_tag = ''
    methods_namespace = ''
    slug_prefix = ''

    fetched = models.DateTimeField(u'Обновлено', null=True, blank=True, db_index=True)

    objects = models.Manager()

    def _substitute(self, old_instance):
        '''
        Substitute new instance with old one while updating in method Manager.get_or_create_from_instance()
        Can be overrided in child models
        '''
        self.pk = old_instance.pk

        # substitute all valueble fields fom old_instance
        for key, old_value in old_instance.__dict__.items():
            new_value = getattr(self, key)
            if old_value and (new_value is None or new_value == ''):
                setattr(self, key, old_value)

    def parse(self, response):
        '''
        Parse API response and define fields with values
        '''
        for key, value in response.items():
            key = key.lower()
            if key == self.remote_pk_field:
                key = self.remote_pk_local_field
#                value = int(value)

            try:
                field = self._meta.get_field(key)
            except FieldDoesNotExist:
                log.debug('Field with name "%s" doesn\'t exists in the model %s' % (key, self.__class__.__name__))
                continue

            if isinstance(field, models.IntegerField) and value:
                try:
                    value = int(value)
                except:
                    pass
            elif isinstance(field, models.FloatField) and value:
                try:
                    value = float(value)
                except:
                    pass
            elif isinstance(field, models.CharField):
                if isinstance(value, bool):
                    value = ''
                else:
                    try:
                        value = unicode(value)
                    except:
                        pass

            elif isinstance(field, (models.DateTimeField, models.DateField)):

                if isinstance(value, string_types) and len(value) >= 10:
                    try:
                        if len(value) == 19:
                            value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]), int(
                                value[11:13]), int(value[14:16]), int(value[17:19]))
                        elif len(value) == 16:
                            value = datetime(int(value[0:4]), int(value[5:7]), int(
                                value[8:10]), int(value[11:13]), int(value[14:16]))
                        elif len(value) == 10:
                            value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]))
                        value = pytz.timezone('Europe/Moscow').localize(value)
                        assert value.year != 1970
                    except AssertionError:
                        value = None
                else:
                    try:
                        value = int(value)
                        assert value > 0
                        value = datetime.utcfromtimestamp(value).replace(tzinfo=timezone.utc)
                    except:
                        value = None

            if isinstance(field, (models.OneToOneField, models.ForeignKey)) and value:
                rel_class = field.rel.to
                if isinstance(value, dict):
                    value = rel_class().parse(dict(value))
                else:
                    try:
                        value = rel_class.objects.get(pk=value)
                    except rel_class.DoesNotExist:
                        key = key + '_id'

            if isinstance(field, (fields.CommaSeparatedCharField, models.CommaSeparatedIntegerField)) and isinstance(value, list):
                value = ','.join([unicode(v) for v in value])

            setattr(self, key, value)

    def refresh(self):
        """
        Refresh current model with remote data
        """
        objects = self.__class__.remote.fetch_one(**self.refresh_kwargs)
        if isinstance(objects, models.Model):
            new_instance = objects
        elif isinstance(objects, (list, tuple, QuerySet)):
            if len(objects) == 1:
                new_instance = objects[0]
            else:
                raise OdnoklassnikiContentError(
                    "Remote server returned more objects, than expected - %d instead of one. Object details: %s, request details: %s" % (len(objects), self.__dict__, kwargs))

        self.__dict__.update(new_instance.__dict__)

    def get_url(self):
        return 'http://odnoklassniki.ru/%s' % self.slug

    @property
    def refresh_kwargs(self):
        raise NotImplementedError("Property %s.refresh_kwargs should be specified" % self.__class__.__name__)

    @property
    def slug(self):
        raise NotImplementedError("Property %s.slug should be specified" % self.__class__.__name__)


class OdnoklassnikiPKModel(OdnoklassnikiModel):

    class Meta:
        abstract = True

    id = models.BigIntegerField(u'ID', help_text=u'Уникальный идентификатор', primary_key=True)

    @property
    def slug(self):
        return '/'.join([self.slug_prefix, str(self.pk)])



# -*- coding: utf-8 -*-
from django.test import TestCase
import mock

from .api import api_call, OdnoklassnikiApi

GROUP_ID = 53038939046008


class OdnoklassnikiApiTest(TestCase):

    def test_api_instance_singleton(self):

        self.assertEqual(id(OdnoklassnikiApi()), id(OdnoklassnikiApi()))

    def test_get_url_info(self):

        response = api_call('url.getInfo', url='http://www.odnoklassniki.ru/apiok')
        self.assertEqual(response, {u'objectId': GROUP_ID, u'type': u'GROUP'})

    @mock.patch('odnoklassniki.api.Odnoklassniki._request', side_effect=lambda *args, **kwargs: (200, {u'error_data': None, u'error_code': 102, u'error_msg': u'PARAM_SESSION_EXPIRED : Session expired'}))
    @mock.patch('odnoklassniki_api.api.OdnoklassnikiApi.handle_error_code_102')
    def test_error_102(self, request, handle_error):

        response = api_call('url.getInfo', url='http://www.odnoklassniki.ru/apiok')
        self.assertTrue(handle_error.called)



# -*- coding: utf-8 -*-
'''
List of fields from API docs
http://apiok.ru/wiki/display/api/fields
'''

API_REQUEST_FIELDS = {
    'media_topic': [
        'author_ref',
        'created_ms',
        'id',
        'like_summary',
        'media',
        'media_description',
        'media_movie_refs',
        'media_music_track_refs',
        'media_photo_refs',
        'media_reshare',
        'media_text',
        'media_title',
        'media_type',
        'media_url',
    ],
    'feed': [
        'type',
        'date',
        'feed_owner_refs',
        'message',
        'group',
        'actor_refs',
        'mark_as_spam_id',
        'like_summary',
        'photo_main',
        'photo_refs',
        'album_refs',
        'owner_refs',
        'friend_refs',
        'target_refs',
        'post_status',
        'post_refs',
        'present_refs',
        'sender_refs',
        'receiver_refs',
    ],
    'group_album': [
        'aid',
        'author_name',
        'author_type',
        'created',
        'group_id',
        # Next one yeild 'attrs' attribute in OK server response.
        'like_allowed',
        'like_summary',
        'photos_count',
        # Next one present in docs still its meaning is unknown
        'ref',
        'title',
        'user_id',
        # Next 3 are documented. Still yeild no results for group album
        'modify_allowed',
        'delete_allowed',
        'add_photo_allowed',
    ],
    'group_photo': [
        'album_id',
        'author_name',
        'author_type',
        'comments_count',
        'created_ms',
        'discussion_summary',
        'group_id',
        'id',
        'like_count',
        'like_summary',
        # Undocumented alas retuned in resp to group_photo.*
        'liked_it',
        # Next 2 yeild 'attrs' attribute in OK server response.
        'like_allowed',
        'mark_as_spam_allowed',
        # Next one is undocumented. Returned in response to fields: group_photo.*
        'pic1024max',
        'pic1024x768',
        'pic128max',
        'pic128x128',
        'pic180min',
        # Next one present in docs. Still yeilds no results for group photos.getPhotos
        'pic190x190',
        'pic240min',
        'pic320min',
        'pic50x50',
        'pic640x480',
        'standard_height',
        'standard_width',
        'text',
    ],
    'discussion': [
        'object_type',
        'object_id',
        'title',
        'truncated',
        'message',
        'access_restricted',
        'creation_date',
        'last_activity_date',
        'last_user_access_date',
        'new_comments_count',
        'total_comments_count',
        'owner_uid',
        'like_summary',
        'like_allowed',
        'mark_as_spam_allowed',
        'comment_allowed',
        'comment_as_admin_allowed',
        'subscribe_allowed',
        'unsubscribe_allowed',
        'likes_unread',
        'reply_unread',
    ],
    'group': [
        'uid',
        'ref',
        'name',
        'description',
        'shortname',
        'pic_avatar',
        'photo_id',
        'shop_visible_admin',
        'shop_visible_public',
        'add_photoalbum_allowed',
        'change_avatar_allowed',
        'members_count',
        'premium',
        'private',
    ],
    'user': [
        'uid',
        'locale', #(en, lv, ... [language[_territory][.codeset][@modifier]] )
        'first_name',
        'last_name',
        'name',
        'gender',
        'age',
        'birthday', #(yyyy-MM-dd)
        'has_email',
        'location',
        'current_location',
        'current_status', # (текст статуса)
        'current_status_id', # (уникальный идентификатор статуса)
        'current_status_date', # (дата создания статуса)
        'online',
        'last_online',
        'photo_id', # - id главной фотографии
#         'pic_1', # - то же что и pic50x50
#         'pic_2', # - то же что и pic128max
#         'pic_3', # - то же что и pic190x190
#         'pic_4', # - то же что и pic640x480
#         'pic_5', # - то же что и pic128x128
        'pic50x50', # - квадратная аватарка 50x50
        'pic128x128', # - квадратная аватарка 128x128
        'pic128max', # - аватарка, вписанная в квадрат 128x128
        'pic180min', # - аватарка, заресайзенная по минимальной стороне 180
        'pic240min', # - аватарка, заресайзенная по минимальной стороне 240
        'pic320min', # - аватарка, заресайзенная по минимальной стороне 320
        'pic190x190', # - квадратная аватарка 190x190
        'pic640x480', # - аватарка, вписанная в квадрат 640x480
        'pic1024x768', # - аватарка, вписанная в квадрат 1024x768
        'url_profile',
        'url_chat',
        'url_profile_mobile',
        'url_chat_mobile',
        'can_vcall',
        'can_vmail',
#        'email', # GET_EMAIL permission is required
        'allows_anonym_access',
        'allows_messaging_only_for_friends',
        'registered_date',
        'has_service_invisible',
    ],
    'comment': [
        'id',
        'author_id',
        'author_type',
        'author_name',
        'text',
        'date',
        'type',
        'reply_to_comment_id',
        'reply_to_id',
        'reply_to_type',
        'reply_to_name',
        'like_summary',
        'delete_allowed',
        'like_allowed',
        'mark_as_spam_allowed',
        'author_block_allowed',
        'likes_unread',
        'reply_unread',
    ]
}


# from django import forms
# from django.utils.safestring import mark_safe
#
# class AdminImageWidget(forms.FileInput):
#     """
#     A ImageField Widget for admin that shows a thumbnail.
#     """
#
#     def __init__(self, attrs={}):
#         super(AdminImageWidget, self).__init__(attrs)
#
#     def render(self, name, value, attrs=None):
#         output = []
#         if value and hasattr(value, "url"):
#             output.append(('<a target="_blank" href="%s">'
#                            '<img src="%s" style="height: 28px;" /></a> '
#                            % (value.url, value.url)))
#         output.append(super(AdminImageWidget, self).render(name, value, attrs))
#         return mark_safe(u''.join(output))


from django.core.exceptions import ImproperlyConfigured


def get_improperly_configured_field(app_name, decorate_property=False):
    def field(self):
        raise ImproperlyConfigured("Application '%s' not in INSTALLED_APPS" % app_name)
    if decorate_property:
        field = property(field)
    return field
